@@ -63,6 +63,7 @@
 #include "gpm-brightness.h"
 #include "gpm-tray-icon.h"
 #include "gpm-inhibit.h"
+#include "gpm-polkit.h"
 #include "gpm-stock-icons.h"
 #include "gpm-manager.h"
 
@@ -108,6 +109,7 @@ struct GpmManagerPrivate
 	GpmBrightness   *brightness;
 	GpmScreensaver  *screensaver;
 	GpmInhibit	*inhibit;
+	GpmPolkit	*polkit;
 
 	guint32          ac_throttle_id;
 	guint32          dpms_throttle_id;
@@ -121,11 +123,11 @@ struct GpmManagerPrivate
 	GpmWarning	 last_keyboard_warning;
 	GpmWarning	 last_pda_warning;
 
-	gint		 last_primary_percentage_change;
-
 	gboolean	 use_time_to_notify;
 	gboolean	 lid_is_closed;
 	gboolean	 done_notify_fully_charged;
+	gboolean	 enable_beeping;
+	gboolean	 ignore_inhibits;
 
 	time_t		 last_resume_event;
 	int		 suppress_policy_timeout;
@@ -139,8 +141,6 @@ struct GpmManagerPrivate
 	int		 very_low_time;
 	int		 critical_time;
 	int		 action_time;
-
-	int		 lcd_dim_brightness;
 };
 
 enum {
@@ -195,9 +195,13 @@ static gboolean
 gpm_manager_is_policy_timout_valid (GpmManager *manager,
 				    const char *action)
 {
+	char *message;
 	if ((time (NULL) - manager->priv->last_resume_event) <=
 	    manager->priv->suppress_policy_timeout) {
-		gpm_debug ("Skipping suppressed %s", action);
+		message = g_strdup_printf ("Skipping suppressed %s", action);
+		gpm_info_event_log (manager->priv->info,
+				    GPM_GRAPH_EVENT_NOTIFICATION, message);
+		g_free (message);
 		return FALSE;
 	}
 	return TRUE;
@@ -223,7 +227,11 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 
 	action_ok = gpm_inhibit_check (manager->priv->inhibit);
 	if (! action_ok) {
-		title = g_strdup_printf ("Request to %s", action);
+		if (manager->priv->ignore_inhibits) {
+			gpm_debug ("Inhibit ignored through gconf policy!");
+			return TRUE;
+		}
+		title = g_strdup_printf (_("Request to %s"), action);
 		GString *message = g_string_new ("");
 		gpm_inhibit_get_message (manager->priv->inhibit, message, action);
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
@@ -232,6 +240,9 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 				      GPM_NOTIFY_TIMEOUT_LONG,
 				      GTK_STOCK_DIALOG_WARNING,
 				      GPM_NOTIFY_URGENCY_NORMAL);
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
 		g_string_free (message, TRUE);
 		g_free (title);
 	}
@@ -239,7 +250,7 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 }
 
 /**
- * gpm_manager_can_suspend:
+ * gpm_manager_allowed_suspend:
  * @manager: This manager class instance
  * @can: If we can suspend
  *
@@ -247,17 +258,23 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
  * checks gconf to see if we are allowed to suspend this computer.
  **/
 gboolean
-gpm_manager_can_suspend (GpmManager *manager,
-			 gboolean   *can,
-			 GError    **error)
+gpm_manager_allowed_suspend (GpmManager *manager,
+			     gboolean   *can,
+			     GError    **error)
 {
-	gboolean gconf_policy;
+	gboolean gconf_ok;
+	gboolean polkit_ok = TRUE;
+	gboolean hal_ok = FALSE;
 	g_return_val_if_fail (can, FALSE);
 
 	*can = FALSE;
-	gconf_policy = gconf_client_get_bool (manager->priv->gconf_client,
+	gconf_ok = gconf_client_get_bool (manager->priv->gconf_client,
 					      GPM_PREF_CAN_SUSPEND, NULL);
-	if ( gconf_policy && gpm_hal_can_suspend () ) {
+	hal_ok = gpm_hal_can_suspend ();
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-suspend");
+#endif
+	if ( gconf_ok && hal_ok && polkit_ok ) {
 		*can = TRUE;
 	}
 
@@ -265,7 +282,7 @@ gpm_manager_can_suspend (GpmManager *manager,
 }
 
 /**
- * gpm_manager_can_hibernate:
+ * gpm_manager_allowed_hibernate:
  * @manager: This manager class instance
  * @can: If we can hibernate
  *
@@ -273,40 +290,69 @@ gpm_manager_can_suspend (GpmManager *manager,
  * checks gconf to see if we are allowed to hibernate this computer.
  **/
 gboolean
-gpm_manager_can_hibernate (GpmManager *manager,
-			   gboolean   *can,
-			   GError    **error)
+gpm_manager_allowed_hibernate (GpmManager *manager,
+			       gboolean   *can,
+			       GError    **error)
 {
-	gboolean gconf_policy;
+	gboolean gconf_ok;
+	gboolean polkit_ok = TRUE;
+	gboolean hal_ok = FALSE;
 	g_return_val_if_fail (can, FALSE);
 
 	*can = FALSE;
-	gconf_policy = gconf_client_get_bool (manager->priv->gconf_client,
+	gconf_ok = gconf_client_get_bool (manager->priv->gconf_client,
 					      GPM_PREF_CAN_HIBERNATE, NULL);
-	if ( gconf_policy && gpm_hal_can_hibernate () ) {
+	hal_ok = gpm_hal_can_hibernate ();
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-hibernate");
+#endif
+	if ( gconf_ok && hal_ok && polkit_ok ) {
 		*can = TRUE;
 	}
-
 	return TRUE;
 }
 
 /**
- * gpm_manager_can_shutdown:
+ * gpm_manager_allowed_shutdown:
  * @manager: This manager class instance
  * @can: If we can shutdown
  *
  * Stub function -- TODO.
  **/
 gboolean
-gpm_manager_can_shutdown (GpmManager *manager,
-			  gboolean   *can,
-			  GError    **error)
+gpm_manager_allowed_shutdown (GpmManager *manager,
+			      gboolean   *can,
+			      GError    **error)
 {
-	if (can) {
-		*can = FALSE;
+	*can = FALSE;
+	gboolean polkit_ok = TRUE;
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-shutdown");
+#endif
+	if ( polkit_ok ) {
+		*can = TRUE;
 	}
-	/* FIXME: check other stuff */
-	if (can) {
+	return TRUE;
+}
+
+/**
+ * gpm_manager_allowed_reboot:
+ * @manager: This manager class instance
+ * @can: If we can reboot
+ *
+ * Stub function -- TODO.
+ **/
+gboolean
+gpm_manager_allowed_reboot (GpmManager *manager,
+			    gboolean   *can,
+			    GError    **error)
+{
+	*can = FALSE;
+	gboolean polkit_ok = TRUE;
+#ifdef HAVE_POLKIT
+	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-reboot");
+#endif
+	if ( polkit_ok ) {
 		*can = TRUE;
 	}
 	return TRUE;
@@ -617,7 +663,9 @@ change_power_policy (GpmManager *manager,
 		power_save = gconf_client_get_bool (client, GPM_PREF_BATTERY_LOWPOWER, NULL);
 	}
 
-	gpm_brightness_level_dim (manager->priv->brightness, brightness);
+	gpm_brightness_set_level_std (manager->priv->brightness, brightness);
+	gpm_brightness_set (manager->priv->brightness);
+
 	gpm_hal_enable_power_save (power_save);
 	update_ac_throttle (manager, on_ac);
 
@@ -819,27 +867,62 @@ manager_policy_do (GpmManager *manager,
 /**
  * gpm_manager_get_on_ac:
  * @manager: This manager class instance
- * @on_ac: TRUE if we are on AC power
+ * @retval: TRUE if we are on AC power
  **/
 gboolean
 gpm_manager_get_on_ac (GpmManager  *manager,
-			gboolean   *on_ac,
+			gboolean   *retval,
 			GError    **error)
 {
+	gboolean on_ac;
+
 	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
 
-	if (on_ac) {
-		gpm_power_get_on_ac (manager->priv->power, on_ac, error);
+	if (retval == NULL) {
+		return FALSE;
 	}
 
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, error);
+	*retval = on_ac;
+
 	return TRUE;
 }
 
+/**
+ * gpm_manager_get_low_power_mode:
+ * @manager: This manager class instance
+ * @retval: TRUE if we are on low power mode
+ **/
+gboolean
+gpm_manager_get_low_power_mode (GpmManager  *manager,
+				gboolean    *retval,
+				GError     **error)
+{
+	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
+
+	if (retval == NULL) {
+		return FALSE;
+	}
+
+	gboolean on_ac;
+	gboolean power_save;
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, error);
+	if (on_ac) {
+		power_save = gconf_client_get_bool (manager->priv->gconf_client,
+						    GPM_PREF_AC_LOWPOWER, NULL);
+	} else {
+		power_save = gconf_client_get_bool (manager->priv->gconf_client,
+						    GPM_PREF_BATTERY_LOWPOWER, NULL);
+	}
+	*retval = power_save;
+
+	return TRUE;
+}
 /**
  * gpm_manager_set_dpms_mode:
  * @manager: This manager class instance
  * @mode: The DPMS mode, e.g. GPM_DPMS_MODE_STANDBY
- * Return value: TRUE if we could set the GPMS mode OK.
+ * Return value: TRUE if we could set the DPMS mode OK.
  **/
 gboolean
 gpm_manager_set_dpms_mode (GpmManager *manager,
@@ -888,7 +971,7 @@ gpm_manager_get_dpms_mode (GpmManager  *manager,
 }
 
 /**
- * gpm_manager_inhibit_inactive_sleep:
+ * gpm_manager_inhibit:
  * @manager: This manager class instance
  * @application: The application that sent the request, e.g. "Nautilus"
  * @reason: The reason given to inhibit, e.g. "Copying files"
@@ -898,11 +981,11 @@ gpm_manager_get_dpms_mode (GpmManager  *manager,
  * idle action suspend from happening.
  **/
 void
-gpm_manager_inhibit_inactive_sleep (GpmManager	*manager,
-				    const char	*application,
-				    const char	*reason,
-				    DBusGMethodInvocation *context,
-				    GError    **error)
+gpm_manager_inhibit (GpmManager	*manager,
+		     const char	*application,
+		     const char	*reason,
+		     DBusGMethodInvocation *context,
+		     GError    **error)
 {
 	const char* connection = dbus_g_method_get_sender (context);
 	int cookie;
@@ -911,7 +994,7 @@ gpm_manager_inhibit_inactive_sleep (GpmManager	*manager,
 }
 
 /**
- * gpm_manager_allow_inactive_sleep:
+ * gpm_manager_uninhibit:
  * @manager: This manager class instance
  * @cookie: The application cookie, e.g. 17534
  * @context: The context we are talking to
@@ -920,10 +1003,10 @@ gpm_manager_inhibit_inactive_sleep (GpmManager	*manager,
  * idle action suspend to happen.
  **/
 void
-gpm_manager_allow_inactive_sleep (GpmManager	 *manager,
-				  int		  cookie,
-				  DBusGMethodInvocation *context,
-				  GError	**error)
+gpm_manager_uninhibit (GpmManager	 *manager,
+		       int		  cookie,
+		       DBusGMethodInvocation *context,
+		       GError		**error)
 {
 	const char* connection = dbus_g_method_get_sender (context);
 	gpm_inhibit_remove (manager->priv->inhibit, connection, cookie);
@@ -943,7 +1026,7 @@ gpm_manager_shutdown (GpmManager *manager,
 	gboolean allowed;
 	gboolean ret;
 
-	gpm_manager_can_shutdown (manager, &allowed, NULL);
+	gpm_manager_allowed_shutdown (manager, &allowed, NULL);
 	if (! allowed) {
 		gpm_warning ("Cannot shutdown");
 		g_set_error (error,
@@ -957,13 +1040,45 @@ gpm_manager_shutdown (GpmManager *manager,
 				   GNOME_SAVE_GLOBAL,
 				   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
 
-	/* FIXME: make this async? */
 	gpm_hal_shutdown ();
 	ret = TRUE;
 
 	return ret;
 }
 
+/**
+ * gpm_manager_reboot:
+ * @manager: This manager class instance
+ *
+ * Reboots the computer, saving the session if possible.
+ **/
+static gboolean
+gpm_manager_reboot (GpmManager *manager,
+		    GError    **error)
+{
+	gboolean allowed;
+	gboolean ret;
+
+	gpm_manager_allowed_reboot (manager, &allowed, NULL);
+	if (! allowed) {
+		gpm_warning ("Cannot reboot");
+		g_set_error (error,
+			     GPM_MANAGER_ERROR,
+			     GPM_MANAGER_ERROR_GENERAL,
+			     "Cannot reboot");
+		return FALSE;
+	}
+
+	gnome_client_request_save (gnome_master_client (),
+				   GNOME_SAVE_GLOBAL,
+				   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
+
+	gpm_hal_reboot ();
+	ret = TRUE;
+
+	return ret;
+}
+
 /**
  * gpm_manager_hibernate:
  * @manager: This manager class instance
@@ -982,7 +1097,7 @@ gpm_manager_hibernate (GpmManager *manager,
 	gboolean ret;
 	gboolean do_lock;
 
-	gpm_manager_can_hibernate (manager, &allowed, NULL);
+	gpm_manager_allowed_hibernate (manager, &allowed, NULL);
 
 	if (! allowed) {
 		gpm_warning ("Cannot hibernate");
@@ -1001,7 +1116,6 @@ gpm_manager_hibernate (GpmManager *manager,
 
 	gpm_networkmanager_sleep ();
 
-	/* FIXME: make this async? */
 	ret = gpm_hal_hibernate ();
 	manager_explain_reason (manager, GPM_GRAPH_EVENT_RESUME,
 				_("Resuming computer"), NULL);
@@ -1065,7 +1179,7 @@ gpm_manager_suspend (GpmManager *manager,
 	GpmPowerStatus status;
 	char *message;
 
-	gpm_manager_can_suspend (manager, &allowed, NULL);
+	gpm_manager_allowed_suspend (manager, &allowed, NULL);
 
 	if (! allowed) {
 		gpm_warning ("Cannot suspend");
@@ -1197,6 +1311,21 @@ gpm_manager_shutdown_dbus_method (GpmManager *manager,
 	return gpm_manager_shutdown (manager, error);
 }
 
+/**
+ * gpm_manager_reboot_dbus_method:
+ * @manager: This manager class instance
+ **/
+gboolean
+gpm_manager_reboot_dbus_method (GpmManager *manager,
+				GError    **error)
+{
+	/* FIXME: From where? */
+	manager_explain_reason (manager, GPM_GRAPH_EVENT_NOTIFICATION,
+				_("Rebooting computer"),
+				_("the DBUS method Reboot() was invoked"));
+	return gpm_manager_reboot (manager, error);
+}
+
 /**
  * idle_changed_cb:
  * @idle: The idle class instance
@@ -1237,15 +1366,15 @@ idle_changed_cb (GpmIdle    *idle,
 			gpm_debug ("Unable to set DPMS active: %s", error->message);
 		}
 
-		/* Should we dim the screen? */
+		/* Should we resume the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
-							GPM_PREF_IDLE_DIM_SCREEN, NULL);
+						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
 		if (do_laptop_dim) {
 			/* resume to the previous brightness */
 			manager_explain_reason (manager, GPM_GRAPH_EVENT_SCREEN_RESUME,
 						_("Screen resume"),
 						_("idle mode ended"));
-			gpm_brightness_level_resume (manager->priv->brightness);
+			gpm_brightness_undim (manager->priv->brightness);
 		}
 
 		/* sync timeouts */
@@ -1256,35 +1385,22 @@ idle_changed_cb (GpmIdle    *idle,
 
 		gpm_debug ("Idle state changed: SESSION");
 
-		/* Activate display power management */
+		/* activate display power management */
 		error = NULL;
 		gpm_dpms_set_active (manager->priv->dpms, TRUE, &error);
 		if (error) {
 			gpm_debug ("Unable to set DPMS active: %s", error->message);
 		}
 
-		/* Should we resume the screen? */
+		/* Should we dim the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
 						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
 		if (do_laptop_dim) {
-			int dim_br;
-			int current_br;
-			dim_br = manager->priv->lcd_dim_brightness;
-			current_br = gpm_brightness_level_get (manager->priv->brightness);
-			if (current_br < dim_br) {
-				/* If the current brightness is less than the dim
-				 * brightness then just use the lowest brightness
-				 * so that we don't *increase* in brightness on idle.
-				 * See #338630 for more details */
-				gpm_warning ("Current brightness is %i, dim brightness is %i.",
-					     current_br, dim_br);
-				dim_br = current_br;
-			}
-			/* Save this brightness and dim the screen, fixes #328564 */
+			/* Dim the screen, fixes #328564 */
 			manager_explain_reason (manager, GPM_GRAPH_EVENT_SCREEN_DIM,
 						_("Screen dim"),
 						_("idle mode started"));
-			gpm_brightness_level_save (manager->priv->brightness, dim_br);
+			gpm_brightness_dim (manager->priv->brightness);
 		}
 
 		/* sync timeouts */
@@ -1441,12 +1557,12 @@ lid_button_pressed (GpmManager	 *manager,
 	if (state) {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_LID_CLOSED,
-				    "The laptop lid has been closed");
+				    _("The laptop lid has been closed"));
 		gpm_debug ("lid button CLOSED");
 	} else {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_LID_OPENED,
-				    "The laptop lid has been re-opened");
+				    _("The laptop lid has been re-opened"));
 		gpm_debug ("lid button OPENED");
 	}
 
@@ -1460,7 +1576,6 @@ lid_button_pressed (GpmManager	 *manager,
 	update_lid_throttle (manager, manager->priv->lid_is_closed);
 
 	if (state) {
-
 		if (on_ac) {
 			gpm_debug ("Performing AC policy");
 			manager_policy_do (manager,
@@ -1535,11 +1650,11 @@ power_button_pressed_cb (GpmPower   *power,
 
 	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_UP) == 0) ||
 		   (strcmp (type, GPM_BUTTON_BRIGHT_UP_DEP) == 0)) {
-		gpm_brightness_level_up (manager->priv->brightness);
+		gpm_brightness_up (manager->priv->brightness);
 
 	} else if ((strcmp (type, GPM_BUTTON_BRIGHT_DOWN) == 0) ||
 		   (strcmp (type, GPM_BUTTON_BRIGHT_DOWN_DEP) == 0)) {
-		gpm_brightness_level_down (manager->priv->brightness);
+		gpm_brightness_down (manager->priv->brightness);
 
 	} else if (strcmp (type, GPM_BUTTON_LOCK) == 0) {
 		gpm_screensaver_lock (manager->priv->screensaver);
@@ -1587,19 +1702,23 @@ power_on_ac_changed_cb (GpmPower   *power,
 	if (on_ac) {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_ON_AC,
-				    "AC adapter inserted");
+				    _("AC adapter inserted"));
 	} else {
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_ON_BATTERY,
-				    "AC adapter removed");
+				    _("AC adapter removed"));
 	}
 
 	/* We do the lid close on battery action if the ac_adapter is removed
 	   when the laptop is closed and on battery. Fixes #331655 */
-	if ( (! on_ac) && manager->priv->lid_is_closed) {
+	gboolean event_when_closed;
+	event_when_closed = gconf_client_get_bool (manager->priv->gconf_client,
+						   GPM_PREF_BATT_EVENT_WHEN_CLOSED, NULL);
+	if (event_when_closed && (!on_ac) && manager->priv->lid_is_closed) {
 		manager_policy_do (manager,
 				   GPM_PREF_BATTERY_BUTTON_LID,
-				   _("the lid has been closed, and the ac adapter removed"));
+				   _("the lid has been closed, and the ac adapter "
+				     "removed (and gconf is okay)"));
 	}
 }
 
@@ -1728,7 +1847,7 @@ manager_critical_action_do (GpmManager *manager)
  * and also do the critical actions here.
  **/
 static void
-battery_status_changed_primary (GpmManager	      *manager,
+battery_status_changed_primary (GpmManager     *manager,
 				GpmPowerKind    battery_kind,
 				GpmPowerStatus *battery_status)
 {
@@ -1748,12 +1867,13 @@ battery_status_changed_primary (GpmManager	      *manager,
 		manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	}
 
-	/* We have to track the last percentage, as when we do the transition
-	 * 99 to 100 some laptops report this as charging, some as not-charging.
-	 * This is probably a race. This method should work for both cases. */
-	if (manager->priv->last_primary_percentage_change == 99 &&
-	    battery_status->percentage_charge == 100 &&
-	    ! manager->priv->done_notify_fully_charged) {
+	/* We use the hardware charged state instead of the old 99%->100%
+	 * percentage charge method, as the icon must disappear when this
+	 * notification is shown (if we set to the display policy "charging")
+	 * and also the fact that some ACPI batteries don't make it to 100% */
+	if (manager->priv->done_notify_fully_charged == FALSE && 
+	    gpm_power_battery_is_charged (battery_status)) {
+
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 						     GPM_PREF_NOTIFY_BATTCHARGED, NULL);
 		if (show_notify) {
@@ -1774,12 +1894,10 @@ battery_status_changed_primary (GpmManager	      *manager,
 	/* We only re-enable the fully charged notification when the battery
 	   drops down to 95% as some batteries charge to 100% and then fluctuate
 	   from ~98% to 100%. See #338281 for details */
-	if (manager->priv->last_primary_percentage_change < 95) {
+	if (battery_status->percentage_charge < 95) {
 		manager->priv->done_notify_fully_charged = FALSE;
 	}
 
-	manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
-
 	if (! battery_status->is_discharging) {
 		gpm_debug ("%s is not discharging", gpm_power_kind_to_localised_string (battery_kind));
 		return;
@@ -1811,7 +1929,7 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 	/* Do different warnings for each GPM_WARNING_* */
 	if (warning_type == GPM_WARNING_ACTION) {
-		const char *action;
+		char *action;
 		timeout = GPM_NOTIFY_TIMEOUT_LONG;
 
 		if (! gpm_manager_is_policy_timout_valid (manager, "critical action")) {
@@ -1824,25 +1942,26 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 		/* TODO: we should probably convert to an ENUM type, and use that */
 		if (strcmp (action, ACTION_NOTHING) == 0) {
-			message = _("The battery is below the critical level and "
-				    "this computer will <b>power-off</b> when the "
-				    "battery becomes completely empty.");
+			message = g_strdup (_("The battery is below the critical level and "
+					      "this computer will <b>power-off</b> when the "
+					      "battery becomes completely empty."));
 
 		} else if (strcmp (action, ACTION_SUSPEND) == 0) {
-			message = _("The battery is below the critical level and "
-				    "this computer is about to suspend.<br>"
-				    "<b>NOTE:</b> A small amount of power is required "
-				    "to keep your computer in a suspended state.");
+			message = g_strdup (_("The battery is below the critical level and "
+					      "this computer is about to suspend.<br>"
+					      "<b>NOTE:</b> A small amount of power is required "
+					      "to keep your computer in a suspended state."));
 
 		} else if (strcmp (action, ACTION_HIBERNATE) == 0) {
-			message = _("The battery is below the critical level and "
-				    "this computer is about to hibernate.");
+			message = g_strdup (_("The battery is below the critical level and "
+					      "this computer is about to hibernate."));
 
 		} else if (strcmp (action, ACTION_SHUTDOWN) == 0) {
-			message = _("The battery is below the critical level and "
-				    "this computer is about to shutdown.");
+			message = g_strdup (_("The battery is below the critical level and "
+					      "this computer is about to shutdown."));
 		}
 
+		g_free (action);
 		/* wait 10 seconds for user-panic */
 		g_timeout_add (1000*10, (GSourceFunc) manager_critical_action_do, manager);
 
@@ -1851,8 +1970,8 @@ battery_status_changed_primary (GpmManager	      *manager,
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 						     GPM_PREF_NOTIFY_ACADAPTER, NULL);
 		if (show_notify) {
-			message = g_strdup_printf (_("The AC Power has been unplugged. "
-						      "The system is now using battery power."));
+			message = g_strdup (_("The AC Power has been unplugged. "
+					      "The system is now using battery power."));
 			timeout = GPM_NOTIFY_TIMEOUT_SHORT;
 		}
 
@@ -1874,6 +1993,9 @@ battery_status_changed_primary (GpmManager	      *manager,
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      title, message, timeout,
 				      icon, GPM_NOTIFY_URGENCY_NORMAL);
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_NOTIFICATION,
 				    title);
@@ -1930,7 +2052,7 @@ battery_status_changed_ups (GpmManager	   *manager,
 	manager->priv->last_ups_warning = warning_type;
 
 	if (warning_type == GPM_WARNING_ACTION) {
-		const char *action;
+		char *action;
 
 		if (! gpm_manager_is_policy_timout_valid (manager, "critical action")) {
 			return;
@@ -1955,6 +2077,7 @@ battery_status_changed_ups (GpmManager	   *manager,
 				    "this computer is about to shutdown.");
 		}
 
+		g_free (action);
 		/* TODO: need to add 10 second delay so we get notification */
 		manager_policy_do (manager, GPM_PREF_UPS_CRITICAL,
 				   _("UPS is critically low"));
@@ -1980,6 +2103,9 @@ battery_status_changed_ups (GpmManager	   *manager,
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      title, message, GPM_NOTIFY_TIMEOUT_LONG,
 				      icon, GPM_NOTIFY_URGENCY_NORMAL);
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
 		gpm_info_event_log (manager->priv->info,
 				    GPM_GRAPH_EVENT_NOTIFICATION,
 				    title);
@@ -2063,6 +2189,9 @@ battery_status_changed_misc (GpmManager	    	   *manager,
 			      GPM_NOTIFY_TIMEOUT_LONG,
 			      icon,
 			      GPM_NOTIFY_URGENCY_NORMAL);
+	if (manager->priv->enable_beeping) {
+		gpm_warning_beep ();
+	}
 	gpm_info_event_log (manager->priv->info, GPM_GRAPH_EVENT_NOTIFICATION, title);
 
 	g_free (message);
@@ -2204,25 +2333,27 @@ gconf_key_changed_cb (GConfClient *client,
 
 		if (on_ac) {
 			brightness = gconf_client_get_int (client, GPM_PREF_AC_BRIGHTNESS, NULL);
-			gpm_brightness_level_set (manager->priv->brightness, brightness);
+			gpm_brightness_set_level_std (manager->priv->brightness, brightness);
+			gpm_brightness_set (manager->priv->brightness);
 		}
 
 	} else if (strcmp (entry->key, GPM_PREF_BATTERY_BRIGHTNESS) == 0) {
 
 		if (! on_ac) {
 			brightness = gconf_client_get_int (client, GPM_PREF_BATTERY_BRIGHTNESS, NULL);
-			gpm_brightness_level_set (manager->priv->brightness, brightness);
+			gpm_brightness_set_level_std (manager->priv->brightness, brightness);
+			gpm_brightness_set (manager->priv->brightness);
 		}
 
 	} else if (strcmp (entry->key, GPM_PREF_CAN_SUSPEND) == 0) {
-		gpm_manager_can_suspend (manager, &enabled, NULL);
+		gpm_manager_allowed_suspend (manager, &enabled, NULL);
 		allowed_in_menu = gconf_client_get_bool (manager->priv->gconf_client,
 							 GPM_PREF_SHOW_ACTIONS_IN_MENU, NULL);
 		gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon),
 					      allowed_in_menu && enabled);
 
 	} else if (strcmp (entry->key, GPM_PREF_CAN_HIBERNATE) == 0) {
-		gpm_manager_can_hibernate (manager, &enabled, NULL);
+		gpm_manager_allowed_hibernate (manager, &enabled, NULL);
 		allowed_in_menu = gconf_client_get_bool (manager->priv->gconf_client,
 							 GPM_PREF_SHOW_ACTIONS_IN_MENU, NULL);
 		gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon),
@@ -2231,10 +2362,10 @@ gconf_key_changed_cb (GConfClient *client,
 	} else if (strcmp (entry->key, GPM_PREF_SHOW_ACTIONS_IN_MENU) == 0) {
 		allowed_in_menu = gconf_client_get_bool (manager->priv->gconf_client,
 							 GPM_PREF_SHOW_ACTIONS_IN_MENU, NULL);
-		gpm_manager_can_suspend (manager, &enabled, NULL);
+		gpm_manager_allowed_suspend (manager, &enabled, NULL);
 		gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon),
 					      allowed_in_menu && enabled);
-		gpm_manager_can_hibernate (manager, &enabled, NULL);
+		gpm_manager_allowed_hibernate (manager, &enabled, NULL);
 		gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon),
 						allowed_in_menu && enabled);
 
@@ -2244,10 +2375,19 @@ gconf_key_changed_cb (GConfClient *client,
 				  	      GPM_PREF_POLICY_TIMEOUT, NULL);
 
 	} else if (strcmp (entry->key, GPM_PREF_PANEL_DIM_BRIGHTNESS) == 0) {
-		manager->priv->lcd_dim_brightness =
-			gconf_client_get_int (manager->priv->gconf_client,
-					      GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
+		brightness = gconf_client_get_int (manager->priv->gconf_client,
+						   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
+		gpm_brightness_set_level_dim (manager->priv->brightness, brightness);
 
+	} else if (strcmp (entry->key, GPM_PREF_ENABLE_BEEPING) == 0) {
+		manager->priv->enable_beeping =
+			gconf_client_get_bool (manager->priv->gconf_client,
+					       GPM_PREF_ENABLE_BEEPING, NULL);
+
+	} else if (strcmp (entry->key, GPM_PREF_IGNORE_INHIBITS) == 0) {
+		manager->priv->ignore_inhibits =
+			gconf_client_get_bool (manager->priv->gconf_client,
+					       GPM_PREF_IGNORE_INHIBITS, NULL);
 	}
 }
 
@@ -2296,8 +2436,8 @@ gpm_manager_tray_icon_suspend (GpmManager   *manager,
 		return;
 	}
 	manager_explain_reason (manager,
-				GPM_GRAPH_EVENT_HIBERNATE,
-				_("Susspending computer"),
+				GPM_GRAPH_EVENT_SUSPEND,
+				_("Suspending computer"),
 				_("user clicked suspend from tray menu"));
 	gpm_manager_suspend (manager, NULL);
 }
@@ -2331,8 +2471,7 @@ gpm_manager_tray_icon_show_info (GpmManager   *manager,
 }
 
 /**
- * gpm_manager_init:
- * @manager: This manager class instance
+ * tray_icon_destroyed:
  **/
 static void
 tray_icon_destroyed (GtkObject *object, gpointer user_data)
@@ -2342,6 +2481,29 @@ tray_icon_destroyed (GtkObject *object, gpointer user_data)
 	manager->priv->tray_icon = gpm_tray_icon_new ();
 }
 
+/**
+ * screensaver_auth_request_cb:
+ * @manager: This manager class instance
+ *
+ * Undim the screen when the login screen appears (see #333290)
+ **/
+static void
+screensaver_auth_request_cb (GpmScreensaver *screensaver,
+			     gboolean        auth,
+			     GpmManager     *manager)
+{
+	/* TODO: This may be a bid of a bodge, as we will have multiple
+		 resume requests -- maybe this need a logic cleanup */
+	if (auth) {
+		gpm_debug ("resuming due to auth begin");
+		gpm_brightness_undim (manager->priv->brightness);
+	}
+}
+
+/**
+ * screensaver_connection_changed_cb:
+ * @manager: This manager class instance
+ **/
 static void
 screensaver_connection_changed_cb (GpmScreensaver *screensaver,
 				   gboolean        connected,
@@ -2361,6 +2523,10 @@ screensaver_connection_changed_cb (GpmScreensaver *screensaver,
 	}
 }
 
+/**
+ * gpm_manager_init:
+ * @manager: This manager class instance
+ **/
 static void
 gpm_manager_init (GpmManager *manager)
 {
@@ -2385,6 +2551,8 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (manager->priv->screensaver, "connection-changed",
 			  G_CALLBACK (screensaver_connection_changed_cb), manager);
+	g_signal_connect (manager->priv->screensaver, "auth-request",
+			  G_CALLBACK (screensaver_auth_request_cb), manager);
 
 	/* FIXME: We shouldn't assume the lid is open at startup */
 	manager->priv->lid_is_closed = FALSE;
@@ -2426,6 +2594,9 @@ gpm_manager_init (GpmManager *manager)
 	/* use a class to handle the complex stuff */
 	manager->priv->inhibit = gpm_inhibit_new ();
 
+#ifdef HAVE_POLKIT
+	manager->priv->polkit = gpm_polkit_new ();
+#endif
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
 	g_signal_connect (G_OBJECT (manager->priv->tray_icon), "destroy",
@@ -2433,17 +2604,15 @@ gpm_manager_init (GpmManager *manager)
 
 	gpm_debug ("initialising info infrastructure");
 	manager->priv->info = gpm_info_new ();
-	/* logging system needs access to the power stuff... bit of a bodge */
-	gpm_info_set_power (manager->priv->info, manager->priv->power);
 
 	/* only show the suspend and hibernate icons if we can do the action,
 	   and the policy allows the actions in the menu */
 	allowed_in_menu = gconf_client_get_bool (manager->priv->gconf_client,
 						 GPM_PREF_SHOW_ACTIONS_IN_MENU, NULL);
-	gpm_manager_can_suspend (manager, &enabled, NULL);
+	gpm_manager_allowed_suspend (manager, &enabled, NULL);
 	gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      enabled && allowed_in_menu);
-	gpm_manager_can_hibernate (manager, &enabled, NULL);
+	gpm_manager_allowed_hibernate (manager, &enabled, NULL);
 	gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      enabled && allowed_in_menu);
 
@@ -2478,15 +2647,18 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->last_keyboard_warning = GPM_WARNING_NONE;
 	manager->priv->last_pda_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_warning = GPM_WARNING_NONE;
-	manager->priv->last_primary_percentage_change = 0;
 	manager->priv->done_notify_fully_charged = FALSE;
 
-	/* We don't want to be notified on coldplug if we are on battery power
-	   this should fix #332322 */
+	/* Don't notify on startup if we are on battery power */
 	if (! on_ac) {
 		manager->priv->last_primary_warning = GPM_WARNING_DISCHARGING;
 	}
 
+	/* Don't notify at startup if we are fully charged on AC */
+	if (on_ac) {
+		manager->priv->done_notify_fully_charged = TRUE;
+	}
+
 	/* We can disable this if the ACPI BIOS is fucked, and the
 	   time_remaining is therefore inaccurate or just plain wrong. */
 	use_time = gconf_client_get_bool (manager->priv->gconf_client,
@@ -2534,8 +2706,18 @@ gpm_manager_init (GpmManager *manager)
 							   GPM_PREF_ACTION_TIME, NULL);
 
 	/* Get dim settings */
-	manager->priv->lcd_dim_brightness = gconf_client_get_int (manager->priv->gconf_client,
-								  GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
+	int lcd_dim_brightness;
+	lcd_dim_brightness = gconf_client_get_int (manager->priv->gconf_client,
+						   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
+	gpm_brightness_set_level_dim (manager->priv->brightness, lcd_dim_brightness);
+
+	/* Do we beep? */
+	manager->priv->enable_beeping = gconf_client_get_bool (manager->priv->gconf_client,
+							       GPM_PREF_ENABLE_BEEPING, NULL);
+
+	/* Do we ignore inhibit requests? */
+	manager->priv->ignore_inhibits = gconf_client_get_bool (manager->priv->gconf_client,
+								GPM_PREF_IGNORE_INHIBITS, NULL);
 }
 
 /**
@@ -2560,6 +2742,9 @@ gpm_manager_finalize (GObject *object)
 	g_object_unref (manager->priv->dpms);
 	g_object_unref (manager->priv->idle);
 	g_object_unref (manager->priv->info);
+#ifdef HAVE_POLKIT
+	g_object_unref (manager->priv->polkit);
+#endif
 	g_object_unref (manager->priv->power);
 	g_object_unref (manager->priv->brightness);
 	g_object_unref (manager->priv->tray_icon);
