@@ -122,7 +122,46 @@ gpm_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,BOOLEAN (gpm-marshal.list:3) */
+/* NONE:STRING,STRING,STRING (gpm-marshal.list:3) */
+void
+gpm_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
+                                        GValue       *return_value,
+                                        guint         n_param_values,
+                                        const GValue *param_values,
+                                        gpointer      invocation_hint,
+                                        gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING) (gpointer     data1,
+                                                           gpointer     arg_1,
+                                                           gpointer     arg_2,
+                                                           gpointer     arg_3,
+                                                           gpointer     data2);
+  register GMarshalFunc_VOID__STRING_STRING_STRING callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING_STRING_STRING) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_string (param_values + 1),
+            g_marshal_value_peek_string (param_values + 2),
+            g_marshal_value_peek_string (param_values + 3),
+            data2);
+}
+
+/* NONE:STRING,BOOLEAN (gpm-marshal.list:4) */
 void
 gpm_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
                                   GValue       *return_value,
@@ -159,7 +198,7 @@ gpm_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,STRING,BOOLEAN (gpm-marshal.list:4) */
+/* NONE:STRING,STRING,BOOLEAN (gpm-marshal.list:5) */
 void
 gpm_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
                                          GValue       *return_value,
@@ -198,9 +237,11 @@ gpm_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING (gpm-marshal.list:5) */
+/* NONE:INT (gpm-marshal.list:6) */
 
-/* NONE:INT,LONG,BOOLEAN,BOOLEAN (gpm-marshal.list:6) */
+/* NONE:STRING (gpm-marshal.list:7) */
+
+/* NONE:INT,LONG,BOOLEAN,BOOLEAN (gpm-marshal.list:8) */
 void
 gpm_marshal_VOID__INT_LONG_BOOLEAN_BOOLEAN (GClosure     *closure,
                                             GValue       *return_value,
